-- This script is an incredible conglomeration of embarrasing
-- hacks. Basically, it provides the ability to load in a help
-- file, and scan it for obvious errors, such as missing required
-- keys, having keys of a wrong name, etc.
--
-- If you're generating a help file by hand, you might want to run
-- it through here to make sure there won't be any unwelcome surprises
-- for someone.

-- Keys that can appear in a module
moduleValidKeys = {
    _type = 1, _name = 1, _synopsis = 1, 
	_description = 1, _variables = 1, _functions = 1,
}

-- Keys that must appear in a module
moduleRequiredKeys = {
	_type = 1, _name = 1, _synopsis = 1,
	_variables = 1, _functions = 1,
}

-- Keys that can appear in a function
functionValidKeys = {
	_type = 1, _name = 1, _returns = 1, _errorreturns = 1,
	_args = 1, _synopsis = 1, _description = 1,
}

-- Keys that must appear in a function
functionRequiredKeys = {
	_type = 1, _name = 1, _synopsis = 1,
}

-- Keys that can appear in a variable
variableValidKeys = {
    _type = 1, _name = 1, _synopsis = 1, _description = 1,                
}

-- Keys that must appear in a variable
variableRequiredKeys = {
	_type = 1, _name = 1, _synopsis = 1,
}

-- Checks that the keys contained in checkTable exist in keyTable. 
-- Generates an error if it doesn't. The table in dumpTable is dumped
-- along with the error, to identify the problem entry.
function tableKeysMatchList (checkTable , keyTable , what , errStr , dumpTable)
	for key, ignored in checkTable do
		if keyTable[key] == nil then
			print ("-----")
			foreach (dumpTable , print)
			print ("-----")
			error (what .. errStr .. key)
		end	
	end
end

-- Does some simple checks to make sure a function record is good.
function verifyFunction (moduleName , func)
	if func._name == "function" then error ("Function in " .. moduleName .. " has a name of function.") end
	if func._type ~= "function" then error ("Function " .. moduleName .. "." .. func._name .. " has an incorrect type.") end
	if strbyte (".") ~= strbyte (func._synopsis , -1) then error ("Function " .. moduleName .. "." .. func._name .. " has an invalid synopsis.") end
	if func._description ~= nil and strbyte ("\n") ~= strbyte (func._description , -1) then error ("Function " .. moduleName .. "." .. func._name .. " has an invalid description.") end
end

-- Does some simple checks to make sure a variable record is good.
function verifyVariable (moduleName , var)
	if var._name == "variable" then error ("Variable in " .. moduleName .. " has a name of variable.") end
	if var._type ~= "variable" then error ("Variable " .. moduleName .. "." .. var._name .. " has an incorrect type.") end
	if strbyte (".") ~= strbyte (var._synopsis , -1) then error ("Variable " .. moduleName .. "." .. var._name .. " has an invalid synopsis.") end
	if var._description ~= nil and strbyte ("\n") ~= strbyte (var._description , -1) then error ("Variable " .. moduleName .. "." .. var._name .. " has an invalid description.") end
end

-- Given a table, verifies that it is a valid module. Doesn't return if
-- it isn't.
function verifyModule (modName)
	write ("Verifying " , modName , " : ")

	local module = dofile (modName)

	tableKeysMatchList (module , moduleValidKeys , "Module" , " has unknown key: " , module)
	tableKeysMatchList (moduleRequiredKeys , module , "Module" , " missing required key: " , module)

    if module._name == "module" then error ("Module has a name of module.") end	
	if module._type ~= "module" then error ("Module " .. module._name .. " has an incorrect type.") end

	if module._functions then 
		for ignored,func in module._functions do
			tableKeysMatchList (func , functionValidKeys , "Function" , " has unknown key: " , func)
			tableKeysMatchList (functionRequiredKeys , func , "Function" , " missing required key: " , func)
			verifyFunction (module._name , func)
		end
	end

	if module._variables then
		for ignored, var in module._variables do
			tableKeysMatchList (var , variableValidKeys , "Variable" , " has unknown key: " , var)
			tableKeysMatchList (variableRequiredKeys , var , "Variable" , " missing required key: " , var)
			verifyVariable (module._name , var)
		end
	end

	print ("Module verifies ok")
end

-- Load and verify every help file we know about
verifyModule ("core.hlp")
verifyModule ("system.hlp")
verifyModule ("string.hlp")
verifyModule ("io.hlp")
verifyModule ("math.hlp")
verifyModule ("debug.hlp")
verifyModule ("bitlib.hlp")
verifyModule ("regex.hlp")
verifyModule ("help.hlp")
verifyModule ("extended.hlp")
verifyModule ("Code.hlp")
verifyModule ("FTP.hlp")
verifyModule ("HTTP.hlp")
verifyModule ("SMTP.hlp")
verifyModule ("URL.hlp")
verifyModule ("cl-compat.hlp")
verifyModule ("luasocket.hlp")

